@@ -3,7 +3,7 @@
 #endif
 #include <libintl.h>
 
-#include <stdlib.h>
+#include <wait.h>
 #include <string.h>
 #include <X11/Xlib.h>
 #include <ibus.h>
@@ -14,7 +14,7 @@
 #include "unikey.h"
 #include "vnconv.h"
 
-#if !IBUS_CHECK_VERSION(1,2,99) // ibus version 1.2.99 below have problem with PROP_TYPE_NORMAL, use RADIO instead
+#if !IBUS_CHECK_VERSION(1,2,99) // ibus below version 1.2.99 have problem with PROP_TYPE_NORMAL, use RADIO instead
 #define PROP_TYPE_NORMAL PROP_TYPE_RADIO
 #endif
 
@@ -138,9 +138,8 @@ static void ibus_unikey_engine_class_init(IBusUnikeyEngineClass* klass)
 
 static void ibus_unikey_engine_init(IBusUnikeyEngine* unikey)
 {
-    GValue v = {0};
     gchar* str;
-    gboolean succ;
+    gboolean b;
     guint i;
 
     unikey->preeditstr = new std::string();
@@ -148,113 +147,76 @@ static void ibus_unikey_engine_init(IBusUnikeyEngine* unikey)
 //set default options
     unikey->im = Unikey_IM[0];
     unikey->oc = Unikey_OC[0];
-    unikey->ukopt.freeMarking           = UNIKEY_OPT_FREEMARKING;
-    unikey->ukopt.modernStyle           = UNIKEY_OPT_MODERNSTYLE;
-    unikey->ukopt.macroEnabled          = UNIKEY_OPT_MACROENABLED;
-    unikey->ukopt.useUnicodeClipboard   = 0; // not use
-    unikey->ukopt.alwaysMacro           = 0; // not use
-    unikey->ukopt.spellCheckEnabled     = UNIKEY_OPT_SPELLCHECKENABLED;
-    unikey->ukopt.autoNonVnRestore      = UNIKEY_OPT_AUTONONVNRESTORE;
-    unikey->process_w_at_begin          = UNIKEY_OPT_PROCESSWATBEGIN;
-    unikey->mouse_capture               = UNIKEY_OPT_MOUSECAPTURE;
+    unikey->ukopt.spellCheckEnabled     = DEFAULT_CONF_SPELLCHECK;
+    unikey->ukopt.autoNonVnRestore      = DEFAULT_CONF_AUTONONVNRESTORE;
+    unikey->ukopt.modernStyle           = DEFAULT_CONF_MODERNSTYLE;
+    unikey->ukopt.freeMarking           = DEFAULT_CONF_FREEMARKING;
+    unikey->ukopt.macroEnabled          = DEFAULT_CONF_MACROENABLED;
+    unikey->process_w_at_begin          = DEFAULT_CONF_PROCESSWATBEGIN;
+    unikey->mouse_capture               = DEFAULT_CONF_MOUSECAPTURE;
 
 // read config value
     // read Input Method
-    succ = ibus_config_get_value(config, "engine/Unikey", "InputMethod", &v);
-    if (succ)
+    if (ibus_unikey_config_get_string(config, CONFIG_SECTION, CONFIG_INPUTMETHOD, &str))
     {
-        str = (gchar*)g_value_get_string(&v);
         for (i = 0; i < NUM_INPUTMETHOD; i++)
         {
             if (strcasecmp(str, Unikey_IMNames[i]) == 0)
             {
                 unikey->im = Unikey_IM[i];
+                break;
             }
         }
-        g_value_unset(&v);
     } // end read Input Method
 
     // read Output Charset
-    succ = ibus_config_get_value(config, "engine/Unikey", "OutputCharset", &v);
-    if (succ)
+    if (ibus_unikey_config_get_string(config, CONFIG_SECTION, CONFIG_OUTPUTCHARSET, &str))
     {
-        str = (gchar*)g_value_get_string(&v);
         for (i = 0; i < NUM_OUTPUTCHARSET; i++)
         {
             if (strcasecmp(str, Unikey_OCNames[i]) == 0)
             {
                 unikey->oc = Unikey_OC[i];
+                break;
             }
         }
-        g_value_unset(&v);
     } // end read Output Charset
 
     // read Unikey Option
     // freemarking
-    succ = ibus_config_get_value(config, "engine/Unikey/Options", "FreeMarking", &v);
-    if (succ)
-    {
-        unikey->ukopt.freeMarking = g_value_get_boolean(&v);
-        g_value_unset(&v);
-    }
+    if (ibus_unikey_config_get_boolean(config, CONFIG_SECTION, CONFIG_FREEMARKING, &b))
+        unikey->ukopt.freeMarking = b;
 
     // modernstyle
-    succ = ibus_config_get_value(config, "engine/Unikey/Options", "ModernStyle", &v);
-    if (succ)
-    {
-        unikey->ukopt.modernStyle = g_value_get_boolean(&v);
-        g_value_unset(&v);
-    }
+    if (ibus_unikey_config_get_boolean(config, CONFIG_SECTION, CONFIG_MODERNSTYLE, &b))
+        unikey->ukopt.modernStyle = b;
 
     // macroEnabled
-    succ = ibus_config_get_value(config, "engine/Unikey/Options", "MacroEnabled", &v);
-    if (succ)
-    {
-        unikey->ukopt.macroEnabled = g_value_get_boolean(&v);
-        g_value_unset(&v);
-    }
+    if (ibus_unikey_config_get_boolean(config, CONFIG_SECTION, CONFIG_MACROENABLED, &b))
+        unikey->ukopt.macroEnabled = b;
 
     // spellCheckEnabled
-    succ = ibus_config_get_value(config, "engine/Unikey/Options", "SpellCheckEnabled", &v);
-    if (succ)
-    {
-        unikey->ukopt.spellCheckEnabled = g_value_get_boolean(&v);
-        g_value_unset(&v);
-    }
+    if (ibus_unikey_config_get_boolean(config, CONFIG_SECTION, CONFIG_SPELLCHECK, &b))
+        unikey->ukopt.spellCheckEnabled = b;
 
     // autoNonVnRestore
-    succ = ibus_config_get_value(config, "engine/Unikey/Options", "AutoNonVnRestore", &v);
-    if (succ)
-    {
-        unikey->ukopt.autoNonVnRestore = g_value_get_boolean(&v);
-        g_value_unset(&v);
-    }
+    if (ibus_unikey_config_get_boolean(config, CONFIG_SECTION, CONFIG_AUTORESTORENONVN, &b))
+        unikey->ukopt.autoNonVnRestore = b;
 
     // ProcessWAtBegin
-    succ = ibus_config_get_value(config, "engine/Unikey/Options", "ProcessWAtBegin", &v);
-    if (succ)
-    {
-        unikey->process_w_at_begin = g_value_get_boolean(&v);
-        g_value_unset(&v);
-    }
+    if (ibus_unikey_config_get_boolean(config, CONFIG_SECTION, CONFIG_PROCESSWATBEGIN, &b))
+        unikey->process_w_at_begin = b;
 
     // MouseCapture
-    succ = ibus_config_get_value(config, "engine/Unikey/Options", "MouseCapture", &v);
-    if (succ)
-    {
-        unikey->mouse_capture = g_value_get_boolean(&v);
-        g_value_unset(&v);
-    }
+    if (ibus_unikey_config_get_boolean(config, CONFIG_SECTION, CONFIG_MOUSECAPTURE, &b))
+        unikey->mouse_capture = b;
     // end read Unikey Option
 // end read config value
 
     // load macro
-    if (unikey->ukopt.macroEnabled == 1)
-    {
-        gchar* fn = get_macro_file();
-        UnikeyLoadMacroTable(fn);
-        g_free(fn);
-    }
+    gchar* fn = get_macro_file();
+    UnikeyLoadMacroTable(fn);
+    g_free(fn);
 
     ibus_unikey_engine_create_property_list(unikey);
 }
@@ -311,8 +273,8 @@ static void ibus_unikey_engine_reset(IBusEngine* engine)
         ibus_engine_hide_preedit_text(engine);
         ibus_unikey_engine_commit_string(engine, unikey->preeditstr->c_str());
         unikey->preeditstr->clear();
-        XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, 0); // emulate a mouse move
-        XFlush(dpy);
+        //XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, 0); // emulate a mouse move
+        //XFlush(dpy);
     }
 
     parent_class->reset(engine);
@@ -334,24 +296,24 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
 {
     IBusProperty* prop;
     IBusText* label;
-    GValue v = {0};
     guint i, j;
 
     unikey = (IBusUnikeyEngine*)engine;
 
     // input method active
-    if (strncmp(prop_name, "InputMethod", strlen("InputMethod")) == 0)
+    if (strncmp(prop_name, CONFIG_INPUTMETHOD, strlen(CONFIG_INPUTMETHOD)) == 0)
     {
         for (i=0; i<NUM_INPUTMETHOD; i++)
         {
-            if (strcmp(prop_name + strlen("InputMethod")+1,
+            if (strcmp(prop_name + strlen(CONFIG_INPUTMETHOD)+1,
                        Unikey_IMNames[i]) == 0)
             {
                 unikey->im = Unikey_IM[i];
 
-                g_value_init(&v, G_TYPE_STRING);
-                g_value_set_string(&v, Unikey_IMNames[i]);
-                ibus_config_set_value(config, "engine/Unikey", "InputMethod", &v);
+                ibus_unikey_config_set_string(config,
+                                            CONFIG_SECTION, 
+                                            CONFIG_INPUTMETHOD,
+                                            Unikey_IMNames[i]);
 
                 // update label
                 for (j=0; j<unikey->prop_list->properties->len; j++)
@@ -359,7 +321,7 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
                     prop = ibus_prop_list_get(unikey->prop_list, j);
                     if (prop==NULL)
                         return;
-                    else if (strcmp(prop->key, "InputMethod")==0)
+                    else if (strcmp(prop->key, CONFIG_INPUTMETHOD) == 0)
                     {
                         label = ibus_text_new_from_static_string(Unikey_IMNames[i]);
                         ibus_property_set_label(prop, label);
@@ -385,18 +347,19 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
     } // end input method active
 
     // output charset active
-    else if (strncmp(prop_name, "OutputCharset", strlen("OutputCharset")) == 0)
+    else if (strncmp(prop_name, CONFIG_OUTPUTCHARSET, strlen(CONFIG_OUTPUTCHARSET)) == 0)
     {
         for (i=0; i<NUM_OUTPUTCHARSET; i++)
         {
-            if (strcmp(prop_name+strlen("OutputCharset")+1,
+            if (strcmp(prop_name+strlen(CONFIG_OUTPUTCHARSET)+1,
                        Unikey_OCNames[i]) == 0)
             {
                 unikey->oc = Unikey_OC[i];
 
-                g_value_init(&v, G_TYPE_STRING);
-                g_value_set_string(&v, Unikey_OCNames[i]);
-                ibus_config_set_value(config, "engine/Unikey", "OutputCharset", &v);
+                ibus_unikey_config_set_string(config,
+                                            CONFIG_SECTION, 
+                                            CONFIG_OUTPUTCHARSET,
+                                            Unikey_OCNames[i]);
 
                 // update label
                 for (j=0; j<unikey->prop_list->properties->len; j++)
@@ -404,7 +367,7 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
                     prop = ibus_prop_list_get(unikey->prop_list, j);
                     if (prop==NULL)
                         return;
-                    else if (strcmp(prop->key, "OutputCharset")==0)
+                    else if (strcmp(prop->key, CONFIG_OUTPUTCHARSET)==0)
                     {
                         label = ibus_text_new_from_static_string(Unikey_OCNames[i]);
                         ibus_property_set_label(prop, label);
@@ -418,7 +381,7 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
                     prop = ibus_prop_list_get(unikey->menu_oc, j);
                     if (prop==NULL)
                         return;
-                    else if (strcmp(prop->key, prop_name)==0)
+                    else if (strcmp(prop->key, prop_name) == 0)
                         prop->state = PROP_STATE_CHECKED;
                     else
                         prop->state = PROP_STATE_UNCHECKED;
@@ -430,22 +393,22 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
     } // end output charset active
 
     // spellcheck active
-    else if (strncmp(prop_name, "Spellcheck", strlen("Spellcheck")) == 0)
+    else if (strcmp(prop_name, CONFIG_SPELLCHECK) == 0)
     {
         unikey->ukopt.spellCheckEnabled = !unikey->ukopt.spellCheckEnabled;
+        ibus_unikey_config_set_boolean(config,
+                                    CONFIG_SECTION, 
+                                    CONFIG_SPELLCHECK,
+                                    unikey->ukopt.spellCheckEnabled);
 
-        g_value_init(&v, G_TYPE_BOOLEAN);
-        g_value_set_boolean(&v, unikey->ukopt.spellCheckEnabled);
-        ibus_config_set_value(config, "engine/Unikey/Options", "SpellCheckEnabled", &v);
-
-        // update state of state
+        // update state
         for (j = 0; j < unikey->menu_opt->properties->len ; j++)
         {
             prop = ibus_prop_list_get(unikey->menu_opt, j);
             if (prop == NULL)
                 return;
 
-            else if (strcmp(prop->key, "Spellcheck") == 0)
+            else if (strcmp(prop->key, CONFIG_SPELLCHECK) == 0)
             {
                 prop->state = (unikey->ukopt.spellCheckEnabled == 1)?
                     PROP_STATE_CHECKED:PROP_STATE_UNCHECKED;
@@ -454,98 +417,23 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
         } // end update state
     } // end spellcheck active
 
-    // AutoNonVnRestore active
-    else if (strncmp(prop_name, "AutoNonVnRestore", strlen("AutoNonVnRestore")) == 0)
-    {
-        unikey->ukopt.autoNonVnRestore = !unikey->ukopt.autoNonVnRestore;
-
-        g_value_init(&v, G_TYPE_BOOLEAN);
-        g_value_set_boolean(&v, unikey->ukopt.autoNonVnRestore);
-        ibus_config_set_value(config, "engine/Unikey/Options", "AutoNonVnRestore", &v);
-
-        // update state of state
-        for (j = 0; j < unikey->menu_opt->properties->len ; j++)
-        {
-            prop = ibus_prop_list_get(unikey->menu_opt, j);
-            if (prop == NULL)
-                return;
-
-            else if (strcmp(prop->key, "AutoNonVnRestore") == 0)
-            {
-                prop->state = (unikey->ukopt.autoNonVnRestore == 1)?
-                    PROP_STATE_CHECKED:PROP_STATE_UNCHECKED;
-                break;
-            }
-        } // end update state
-    } // end AutoNonVnRestore active
-
-    // ModernStyle active
-    else if (strncmp(prop_name, "ModernStyle", strlen("ModernStyle")) == 0)
-    {
-        unikey->ukopt.modernStyle = !unikey->ukopt.modernStyle;
-
-        g_value_init(&v, G_TYPE_BOOLEAN);
-        g_value_set_boolean(&v, unikey->ukopt.modernStyle);
-        ibus_config_set_value(config, "engine/Unikey/Options", "ModernStyle", &v);
-
-        // update state of state
-        for (j = 0; j < unikey->menu_opt->properties->len ; j++)
-        {
-            prop = ibus_prop_list_get(unikey->menu_opt, j);
-            if (prop == NULL)
-                return;
-
-            else if (strcmp(prop->key, "ModernStyle") == 0)
-            {
-                prop->state = (unikey->ukopt.modernStyle == 1)?
-                    PROP_STATE_CHECKED:PROP_STATE_UNCHECKED;
-                break;
-            }
-        } // end update state
-    } // end ModernStyle active
-
-    // FreeMarking active
-    else if (strncmp(prop_name, "FreeMarking", strlen("FreeMarking")) == 0)
-    {
-        unikey->ukopt.freeMarking = !unikey->ukopt.freeMarking;
-
-        g_value_init(&v, G_TYPE_BOOLEAN);
-        g_value_set_boolean(&v, unikey->ukopt.freeMarking);
-        ibus_config_set_value(config, "engine/Unikey/Options", "FreeMarking", &v);
-
-        // update state of state
-        for (j = 0; j < unikey->menu_opt->properties->len ; j++)
-        {
-            prop = ibus_prop_list_get(unikey->menu_opt, j);
-            if (prop == NULL)
-                return;
-
-            else if (strcmp(prop->key, "FreeMarking") == 0)
-            {
-                prop->state = (unikey->ukopt.freeMarking == 1)?
-                    PROP_STATE_CHECKED:PROP_STATE_UNCHECKED;
-                break;
-            }
-        } // end update state
-    } // end FreeMarking active
-
     // MacroEnabled active
-    else if (strncmp(prop_name, "MacroEnabled", strlen("MacroEnabled")) == 0)
+    else if (strcmp(prop_name, CONFIG_MACROENABLED) == 0)
     {
         unikey->ukopt.macroEnabled = !unikey->ukopt.macroEnabled;
+        ibus_unikey_config_set_boolean(config,
+                                    CONFIG_SECTION, 
+                                    CONFIG_MACROENABLED,
+                                    unikey->ukopt.macroEnabled);
 
-        g_value_init(&v, G_TYPE_BOOLEAN);
-        g_value_set_boolean(&v, unikey->ukopt.macroEnabled);
-        ibus_config_set_value(config, "engine/Unikey/Options", "MacroEnabled", &v);
-
-        // update state of state
+        // update state
         for (j = 0; j < unikey->menu_opt->properties->len ; j++)
         {
             prop = ibus_prop_list_get(unikey->menu_opt, j);
             if (prop == NULL)
                 return;
 
-            else if (strcmp(prop->key, "MacroEnabled") == 0)
+            else if (strcmp(prop->key, CONFIG_MACROENABLED) == 0)
             {
                 prop->state = (unikey->ukopt.macroEnabled == 1)?
                     PROP_STATE_CHECKED:PROP_STATE_UNCHECKED;
@@ -554,48 +442,24 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
         } // end update state
     } // end MacroEnabled active
 
-    // ProcessWAtBegin active
-    else if (strncmp(prop_name, "ProcessWAtBegin", strlen("ProcessWAtBegin")) == 0)
-    {
-        unikey->process_w_at_begin = !unikey->process_w_at_begin;
-
-        g_value_init(&v, G_TYPE_BOOLEAN);
-        g_value_set_boolean(&v, unikey->process_w_at_begin);
-        ibus_config_set_value(config, "engine/Unikey/Options", "ProcessWAtBegin", &v);
-
-        // update state of state
-        for (j = 0; j < unikey->menu_opt->properties->len ; j++)
-        {
-            prop = ibus_prop_list_get(unikey->menu_opt, j);
-            if (prop == NULL)
-                return;
-
-            else if (strcmp(prop->key, "ProcessWAtBegin") == 0)
-            {
-                prop->state = (unikey->process_w_at_begin == 1)?
-                    PROP_STATE_CHECKED:PROP_STATE_UNCHECKED;
-                break;
-            }
-        } // end update state
-    } // end ProcessWAtBegin active
-
     // MouseCapture active
-    else if (strncmp(prop_name, "MouseCapture", strlen("MouseCapture")) == 0)
+    else if (strcmp(prop_name, CONFIG_MOUSECAPTURE) == 0)
     {
         unikey->mouse_capture = !unikey->mouse_capture;
 
-        g_value_init(&v, G_TYPE_BOOLEAN);
-        g_value_set_boolean(&v, unikey->mouse_capture);
-        ibus_config_set_value(config, "engine/Unikey/Options", "MouseCapture", &v);
+        ibus_unikey_config_set_boolean(config,
+                                    CONFIG_SECTION, 
+                                    CONFIG_MOUSECAPTURE,
+                                    unikey->mouse_capture);
 
-        // update state of state
+        // update state
         for (j = 0; j < unikey->menu_opt->properties->len ; j++)
         {
             prop = ibus_prop_list_get(unikey->menu_opt, j);
             if (prop == NULL)
                 return;
 
-            else if (strcmp(prop->key, "MouseCapture") == 0)
+            else if (strcmp(prop->key, CONFIG_MOUSECAPTURE) == 0)
             {
                 prop->state = (unikey->mouse_capture == 1)?
                     PROP_STATE_CHECKED:PROP_STATE_UNCHECKED;
@@ -606,7 +470,7 @@ static void ibus_unikey_engine_property_activate(IBusEngine* engine,
 
 
     // if Run setup
-    else if (strncmp(prop_name, "RunSetupGUI", strlen("RunSetupGUI")) == 0)
+    else if (strcmp(prop_name, "RunSetupGUI") == 0)
     {
         pthread_t pid;
         pthread_create(&pid, NULL, &thread_run_setup, NULL);
@@ -637,7 +501,7 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
     {
         label = ibus_text_new_from_static_string(Unikey_IMNames[i]);
         tooltip = ibus_text_new_from_static_string(""); // ?
-        sprintf(name, "InputMethod-%s", Unikey_IMNames[i]);
+        sprintf(name, CONFIG_INPUTMETHOD"_%s", Unikey_IMNames[i]);
         prop = ibus_property_new(name,
                                  PROP_TYPE_RADIO,
                                  label,
@@ -658,7 +522,7 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
     {
         label = ibus_text_new_from_static_string(Unikey_OCNames[i]);
         tooltip = ibus_text_new_from_static_string(""); // ?
-        sprintf(name, "OutputCharset-%s", Unikey_OCNames[i]);
+        sprintf(name, CONFIG_OUTPUTCHARSET"_%s", Unikey_OCNames[i]);
         prop = ibus_property_new(name,
                                  PROP_TYPE_RADIO,
                                  label,
@@ -679,7 +543,7 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
     // --create and add spellcheck property
     label = ibus_text_new_from_static_string(_("Enable spell check"));
     tooltip = ibus_text_new_from_static_string(_("If enable, you can decrease mistake when typing"));
-    prop = ibus_property_new("Spellcheck",
+    prop = ibus_property_new(CONFIG_SPELLCHECK,
                              PROP_TYPE_TOGGLE,
                              label,
                              "",
@@ -692,59 +556,10 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
 
     ibus_prop_list_append(unikey->menu_opt, prop);
 
-    // --create and add autononvnrestore property
-    label = ibus_text_new_from_static_string(_("Auto restore keys with invalid words"));
-    tooltip = ibus_text_new_from_static_string(_("When typing a word not in Vietnamese,\nit will auto restore keystroke into original"));
-    prop = ibus_property_new("AutoNonVnRestore",
-                             PROP_TYPE_TOGGLE,
-                             label,
-                             "",
-                             tooltip,
-                             TRUE,
-                             TRUE,
-                             (unikey->ukopt.autoNonVnRestore==1)?
-                             PROP_STATE_CHECKED:PROP_STATE_UNCHECKED,
-                             NULL);
-
-    ibus_prop_list_append(unikey->menu_opt, prop);
-
-    // --create and add modernstyle property
-    label = ibus_text_new_from_static_string(_("Use oà, uý (instead of òa, úy)"));
-    tooltip = ibus_text_new_from_static_string("");
-    prop = ibus_property_new("ModernStyle",
-                             PROP_TYPE_TOGGLE,
-                             label,
-                             "",
-                             tooltip,
-                             TRUE,
-                             TRUE,
-                             (unikey->ukopt.modernStyle==1)?
-                             PROP_STATE_CHECKED:PROP_STATE_UNCHECKED,
-                             NULL);
-
-    ibus_prop_list_append(unikey->menu_opt, prop);
-
-
-    // --create and add freemarking property
-    label = ibus_text_new_from_static_string(_("Allow type with more freedom"));
-    tooltip = ibus_text_new_from_static_string("");
-    prop = ibus_property_new("FreeMarking",
-                             PROP_TYPE_TOGGLE,
-                             label,
-                             "",
-                             tooltip,
-                             TRUE,
-                             TRUE,
-                             (unikey->ukopt.freeMarking==1)?
-                             PROP_STATE_CHECKED:PROP_STATE_UNCHECKED,
-                             NULL);
-
-    ibus_prop_list_append(unikey->menu_opt, prop);
-
     // --create and add macroEnabled property
     label = ibus_text_new_from_static_string(_("Enable Macro"));
     tooltip = ibus_text_new_from_static_string("");
-    prop = ibus_property_new("MacroEnabled",
+    prop = ibus_property_new(CONFIG_MACROENABLED,
                              PROP_TYPE_TOGGLE,
                              label,
                              "",
@@ -757,26 +572,10 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
 
     ibus_prop_list_append(unikey->menu_opt, prop);
 
-    // --create and add ProcessWAtBegin property
-    label = ibus_text_new_from_static_string(_("Process W at word begin"));
-    tooltip = ibus_text_new_from_static_string("");
-    prop = ibus_property_new("ProcessWAtBegin",
-                             PROP_TYPE_TOGGLE,
-                             label,
-                             "",
-                             tooltip,
-                             TRUE,
-                             TRUE,
-                             (unikey->process_w_at_begin==1)?
-                             PROP_STATE_CHECKED:PROP_STATE_UNCHECKED,
-                             NULL);
-
-    ibus_prop_list_append(unikey->menu_opt, prop);
-
     // --create and add MouseCapture property
     label = ibus_text_new_from_static_string(_("Capture mouse event"));
     tooltip = ibus_text_new_from_static_string(_("Auto send PreEdit string to Application when mouse move or click"));
-    prop = ibus_property_new("MouseCapture",
+    prop = ibus_property_new(CONFIG_MOUSECAPTURE,
                              PROP_TYPE_TOGGLE,
                              label,
                              "",
@@ -797,8 +596,8 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
     ibus_prop_list_append(unikey->menu_opt, prop);
 
     // --create and add Launch Setup GUI property
-    label = ibus_text_new_from_static_string(_("Run Setup..."));
-    tooltip = ibus_text_new_from_static_string(_("Run setup utility for IBus-Unikey"));
+    label = ibus_text_new_from_static_string(_("Full setup..."));
+    tooltip = ibus_text_new_from_static_string(_("Full setup utility for IBus-Unikey"));
     prop = ibus_property_new("RunSetupGUI",
                              PROP_TYPE_NORMAL,
                              label,
@@ -822,7 +621,7 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
     }
     label = ibus_text_new_from_static_string(Unikey_IMNames[i]);
     tooltip = ibus_text_new_from_static_string(_("Choose input method"));
-    prop = ibus_property_new("InputMethod",
+    prop = ibus_property_new(CONFIG_INPUTMETHOD,
                              PROP_TYPE_MENU,
                              label,
                              "",
@@ -842,7 +641,7 @@ static void ibus_unikey_engine_create_property_list(IBusUnikeyEngine* unikey)
     }
     label = ibus_text_new_from_static_string(Unikey_OCNames[i]);
     tooltip = ibus_text_new_from_static_string(_("Choose output charset"));
-    prop = ibus_property_new("OutputCharset",
+    prop = ibus_property_new(CONFIG_OUTPUTCHARSET,
                              PROP_TYPE_MENU,
                              label,
                              "",
@@ -1184,10 +983,12 @@ static void* thread_mouse_capture(void* data)
 
 static void* thread_run_setup(void* data)
 {
-    gchar s[1024];
-    strcpy(s, LIBEXECDIR "/ibus-setup-unikey --engine");
-    s[0] = system(s); // for not warning only
-    ibus_quit();
+    int stat;
+    
+    popen(LIBEXECDIR "/ibus-setup-unikey --engine", "r");
+    wait(&stat);
+    if (stat == 0)
+        ibus_quit();
     return NULL;
 }
 
